use crate::prelude::*;
use cryptography::passwords::{check_master_pass, PasswordValidity};

#[inline]
fn process_validity(validity: PasswordValidity) -> Result<(), String> {
    if validity.is_empty() {
        return Ok(());
    }

    let mut error_text = "Your master password is invalid:\n".to_owned();

    if validity.contains(PasswordValidity::NO_LOWERCASE) {
        error_text.push_str("\n* It doesn't have any lowercase characters")
    }
    if validity.contains(PasswordValidity::NO_UPPERCASE) {
        error_text.push_str("\n* It doesn't have any uppercase characters")
    }
    if validity.contains(PasswordValidity::NO_NUMBER) {
        error_text.push_str("\n* It doesn't have any numbers")
    }
    if validity.contains(PasswordValidity::NO_SPECIAL_CHARACTER) {
        error_text.push_str("\n* It doesn't have any special characters")
    }
    if validity.contains(PasswordValidity::TOO_SHORT) {
        error_text.push_str("\n* It is shorter than 8 characters")
    }

    error_text.push_str("\n\nModify your password and send it again");

    Err(error_text)
}

/// Checks that the account with that name exists
#[inline]
async fn check_new_master_pass(
    bot: &Throttle<Bot>,
    msg: &Message,
    password: &str,
) -> crate::Result<Option<Message>> {
    let validity = check_master_pass(password);

    match process_validity(validity) {
        Ok(()) => Ok(None),
        Err(error_text) => {
            let msg = bot.send_message(msg.chat.id, error_text).await?;
            Ok(Some(msg))
        }
    }
}

/// Handles GetNewMasterPass state
pub async fn get_new_master_pass(
    bot: Throttle<Bot>,
    msg: Message,
    db: DatabaseConnection,
    dialogue: MainDialogue,
    next: PackagedHandler<String>,
) -> crate::Result<()> {
    super::generic::generic(
        bot,
        msg,
        db,
        dialogue,
        |bot, msg, _, password| Box::pin(check_new_master_pass(bot, msg, password)),
        "Couldn't get the text of the message. Send the master password again",
        next,
    )
    .await
}
